import pathlib


def create_folder(folder_name):
    """
    Membuat folder.
    Membuat folder secara recursive dengan permission.

    ```py
    create_folder("contoh_membuat_folder")
    create_folder("contoh/membuat/folder/recursive")
    create_folder("./contoh_membuat_folder/secara/recursive")
    ```
    """
    pathlib.Path(folder_name).mkdir(parents=True, exist_ok=True)


if __name__ == "__main__":
    from sys import argv

    try:
        for i in argv[1:]:
            create_folder(i)
    except Exception:
        print("help")



from ..ibuiltins.LINUX import LINUX
from ..ibuiltins.WINDOWS import WINDOWS


def ienv(on_windows=None, on_linux=None):
    """
    Mengambalikan hasil berdasarkan environment dimana program dijalankan

    ```py
    getch = __import__(ienv(on_windows="msvcrt", on_linux="getch"))

    inherit = ienv(
        on_windows=[BaseForWindows, BaseEnv, object],
        on_linux=[SpecialForLinux, BaseForLinux, BaseEnv, object]
    )

    class ExampleIEnv(*inherit):
        pass
    ```
    """
    if WINDOWS:
        return on_windows
    if LINUX:
        return on_linux
    raise Exception("Environment tidak diketahui.")



from .batchmaker import batchmaker
from .calculate import calculate


def batch_calculate(pattern):
    """
    Analisa perhitungan massal.
    Bisa digunakan untuk mencari alternatif terendah/tertinggi/dsb.


    ```python
    iprint(batch_calculate("{1 10} m ** {1 3}"))
    ```
    """
    patterns = batchmaker(pattern)

    c = None
    for i in patterns:
        try:
            c = calculate(i)
        except Exception:
            c = None
        yield (i, c)



import sys

from .is_empty import is_empty


def exit_if_empty(*args):
    """
    Keluar dari program apabila seluruh variabel
    setara dengan empty

    ```py
    var1 = None
    var2 = '0'
    exit_if_empty(var1, var2)
    ```
    """
    for i in args:
        if not is_empty(i):
            return
    sys.exit()



import collections
import sys

iargv = collections.defaultdict()

for k, v in enumerate(sys.argv):
    # Access by Index
    iargv[k] = v

    # Access by Kwargs
    try:
        nk, nv = v.split("=")
        iargv[nk] = nv
    except Exception:
        pass



import pathlib


def iscandir(
    folder_name=".",
    glob_pattern="*",
    recursive=True,
    scan_file=True,
    scan_folder=True,
):
    """
    Mempermudah scandir untuk mengumpulkan folder dan file.

    ```python
    print(iscandir())
    print(list(iscandir("./", recursive=False, scan_file=False)))
    ```
    """
    path_obj = pathlib.Path(folder_name)
    if recursive:
        path_obj = path_obj.rglob(glob_pattern)
    else:
        path_obj = path_obj.glob(glob_pattern)

    for i in path_obj:
        if scan_folder and i.is_dir():
            yield i
        if scan_file and i.is_file():
            yield i



"""
Modul utama pypipr yang berisi semua yg ada dalam pypipr.
"""
from .ibuiltins.avg import *
from .ibuiltins.basename import *
from .ibuiltins.chunk_array import *
from .ibuiltins.ComparePerformance import *
from .ibuiltins.create_folder import *
from .ibuiltins.datetime_from_string import *
from .ibuiltins.datetime_now import *
from .ibuiltins.dict_first import *
from .ibuiltins.dirname import *
from .ibuiltins.exit_if_empty import *
from .ibuiltins.filter_empty import *
from .ibuiltins.get_class_method import *
from .ibuiltins.get_filemtime import *
from .ibuiltins.get_filesize import *
from .ibuiltins.is_empty import *
from .ibuiltins.is_iterable import *
from .ibuiltins.is_valid_url import *
from .ibuiltins.LINUX import *
from .ibuiltins.password_generator import *
from .ibuiltins.random_bool import *
from .ibuiltins.RunParallel import *
from .ibuiltins.set_timeout import *
from .ibuiltins.sets_ordered import *
from .ibuiltins.str_cmp import *
from .ibuiltins.to_str import *
from .ibuiltins.WINDOWS import *
from .iconsole.console_run import *
from .iconsole.input_char import *
from .iconsole.log import *
from .iconsole.print_colorize import *
from .iconsole.print_dir import *
from .iconsole.print_log import *
from .iconsole.text_colorize import *
from .idjango.APIMixinView import *
from .iengineering.batch_calculate import *
from .iengineering.batchmaker import *
from .iengineering.calculate import *
from .iengineering.PintUreg import *
from .iengineering.PintUregQuantity import *
from .iflow.github_pull import *
from .iflow.github_push import *
from .iflow.github_user import *
from .iflow.pip_freeze_without_version import *
from .iflow.poetry_publish import *
from .iflow.poetry_update_version import *
from .ifunctions.idumps import *
from .ifunctions.idumps_html import *
from .ifunctions.ienv import *
from .ifunctions.iexec import *
from .ifunctions.ijoin import *
from .ifunctions.iloads import *
from .ifunctions.iloads_html import *
from .ifunctions.iopen import *
from .ifunctions.iprint import *
from .ifunctions.irange import *
from .ifunctions.ireplace import *
from .ifunctions.iscandir import *
from .ifunctions.isplit import *



from ..ibuiltins.WINDOWS import WINDOWS
from ..iconsole.console_run import console_run
from ..ifunctions.ijoin import ijoin
from ..ifunctions.iopen import iopen


def pip_freeze_without_version(filename=None):
    """
    Memberikan list dari dependencies yang terinstall tanpa version.
    Bertujuan untuk menggunakan Batteries Included Python.

    ```py
    print(pip_freeze_without_version())
    ```
    """
    run = console_run("PIP Freeze", "pip list --format=freeze", capture_output=True)

    text = run.stdout
    if WINDOWS:
        text = text.decode()

    res = ijoin((i.split("=")[0] for i in text.splitlines()), separator="\n")

    if filename:
        return iopen(filename, data=res)

    return res



from ..iconsole.console_run import console_run


def poetry_update_version(mayor=False, minor=False, patch=False):
    """
    Update versi pada pyproject.toml menggunakan poetry

    ```py
    poetry_update_version()
    ```
    """
    if mayor:
        console_run("Update version mayor", "poetry version mayor")
    if minor:
        console_run("Update version minor", "poetry version minor")
    if patch:
        console_run("Update version patch", "poetry version patch")



from .is_iterable import is_iterable
from .to_str import to_str


def filter_empty(iterable, zero_is_empty=True, str_strip=True):
    """
    Mengembalikan iterabel yang hanya memiliki nilai

    ```python
    var = [1, None, False, 0, "0", True, {}, ['eee']]
    print(filter_empty(var))
    ```
    """
    for i in iterable:
        if i == 0 and zero_is_empty:
            continue
        if isinstance(i, str) and str_strip:
            i = i.strip()
        if not is_iterable(i) and not to_str(i):
            continue
        yield i



import datetime
import zoneinfo


def datetime_now(timezone=None):
    """
    Memudahkan dalam membuat Datetime untuk suatu timezone tertentu

    ```python
    print(datetime_now("Asia/Jakarta"))
    print(datetime_now("GMT"))
    print(datetime_now("Etc/GMT+7"))
    ```
    """
    tz = zoneinfo.ZoneInfo(timezone) if timezone else None
    return datetime.datetime.now(tz)



def chunck_array(array, size, start=0):
    """
    Membagi array menjadi potongan-potongan dengan besaran yg diinginkan

    ```python
    array = [2, 3, 12, 3, 3, 42, 42, 1, 43, 2, 42, 41, 4, 24, 32, 42, 3, 12, 32, 42, 42]
    print(chunck_array(array, 5))
    print(list(chunck_array(array, 5)))
    ```
    """
    for i in range(start, len(array), size):
        yield array[i : i + size]



import colorama


def print_colorize(
    text,
    color=colorama.Fore.GREEN,
    bright=colorama.Style.BRIGHT,
    color_end=colorama.Style.RESET_ALL,
    text_start="",
    text_end="\n",
):
    """
    Print text dengan warna untuk menunjukan text penting

    ```py
    print_colorize("Print some text")
    print_colorize("Print some text", color=colorama.Fore.RED)
    ```
    """
    print(f"{text_start}{color + bright}{text}{color_end}", end=text_end, flush=True)
